#finding2_grapher.py
#script that calculates and graphs the interaction beta for several times
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import statsmodels.formula.api as smf
import gc

def increment_dates(dist_mo, dist_year, times):
    date_months = []
    date_years = []
    for t in times:
        test_month = dist_mo+t
        if (24 >= test_month > 12) :
            month = test_month-12
            year = dist_year+1
        elif (36 >= test_month > 24):
            month = test_month-24
            year = dist_year+2
        elif (48 >= test_month > 36):
            month = test_month-36
            year = dist_year+3
        elif (60 >= test_month > 48):
            month = test_month-48
            year = dist_year+4
        elif (test_month > 60):
            month = test_month-60
            year = dist_year+5
        else:
            month = test_month
            year = dist_year
        date_months.append(month)
        date_years.append(year)
    return date_months, date_years

df = pd.read_csv('Desktop/Desktop/correct_full_final_fin_nodupe.csv')

hurricanes =['hermine','matthew','harvey', 'irma','florence','michael','dorian']
dist_mo = [8,10,8,9,9,10,9]
dist_yr = [2016,2016,2017,2017,2018,2018,2019]
cutoff_dates = list(zip(dist_mo, dist_yr))
dict_zip = dict(zip(hurricanes, cutoff_dates))

long_model_log = 'log_light ~ fema_int + post+ fema_treat + urban+state_FIPS+per_female+per_white+per_black+ per_aa + other_race+ u20+middle_age +elderly +employment+population+income'
long_model_log_ia = 'log_light ~ fema_int_ia + post+ fema_IA + urban+state_FIPS+per_female+per_white+per_black+ per_aa + other_race + u20+ middle_age + elderly +employment+population+income'


dflist = []
n_beta_list=[]
ia_beta_list=[]
for h in hurricanes:
    disaster_month = dict_zip[h][0]
    disaster_year = dict_zip[h][1]
    month_list = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48]
    #time_names = ['1_month','2_months','3_months','4_months','5_months','6_months','7_months','8_months','9_months','10_months','11_months','12_months','18_months','24_months','30_months','36_months']
    months, years = increment_dates(disaster_month, disaster_year, month_list)
    dates = list(zip(months, years))
    a = pd.DataFrame(dates, columns = ['month','year'])
    a['hurricane'] = h
    dflist.append(a)
    for d in dates:
            newdf = df[df.hurricane == h]
            newdf['marker'] = np.where(((newdf['month']<=d[0]) & (newdf['year']==d[1])) |(newdf['year']<d[1]), 1, 0)
            newdf = newdf[(newdf['marker']==1)].fillna(0)
            mod = smf.ols(formula =long_model_log, data =newdf)
            res = mod.fit()
            print ('this model is for hurricane: '+h.upper())
            print ('this model is date: '+str(d))
            print ('This is the NORMAL MODEL')
            n_beta_list.append(res.params['fema_int'])
            mod_ia = smf.ols(formula =long_model_log_ia, data =newdf)
            res_ia = mod_ia.fit()
            print ('This is the INDIVIDUAL ASSISSTANCE MODEL')
            ia_beta_list.append(res_ia.params['fema_int_ia'])
del df, dict_zip
gc.collect()

dfp = pd.concat(dflist)
dfp['betas'] = n_beta_list
dfp['betas_ia'] = ia_beta_list
dfp['time'] = month_list*7
zeros = pd.DataFrame({'month': [x-1 for x in dist_mo],
    'year': dist_yr,
    'hurricane': hurricanes,
    'betas': [0 for x in dist_mo],
    'betas_ia': [0 for x in dist_mo],
    'time': [0 for x in dist_mo]}
    )
dfp = dfp.append(zeros)
dfp['per_beta'] = dfp['betas']*100
dfp['per_beta_ia'] = dfp['betas_ia']*100
dfp= dfp[dfp.year<=2020]
dfp.reset_index(drop=True, inplace=True)


dfp['str_date'] = dfp['year'].astype(str)+'-'+dfp['month'].astype(str)
dfp['date'] = pd.to_datetime(dfp['str_date'])

dfia = dfp.copy()
dfcopy = dfp.copy()
dfcopy = dfcopy.drop(dfcopy[dfcopy.hurricane=='irma'].index)
dfia = dfia.drop(dfia[dfia.hurricane=='dorian'].index)

hurricanes =['hermine','matthew','harvey', 'irma','florence','michael','dorian']
tcs = [59362714.8, 196323879.86, 2362663802.25, 698546957.9, 321465576.97, 669170843.4, 33016496.57]
ics = [8693303.61, 16387261.05, 1656898380.5, 286163774.37, 54907630.30, 57427696.54, np.nan]
total_cost_dict = dict(zip(hurricanes, tcs))
individ_cost_dict = dict(zip(hurricanes, ics))

dfcopy['total_cost'] = dfcopy['hurricane'].map(total_cost_dict)
dfia['individual_cost'] = dfia['hurricane'].map(individ_cost_dict)

dfcopy['tc_norm'] = dfcopy['per_beta']/dfcopy['total_cost']
dfia['ic_norm'] = dfia['per_beta_ia']/dfia['individual_cost']


dfp_palette = sns.color_palette('muted', 7) #muted, winter, Purples, PuBu, PuRd, BuGn
dfp_palette.pop(3)
dfia_palette = sns.color_palette('muted', 7)
dfia_palette.pop(6)

dash_list_norm = sns._core.unique_dashes(7)
dash_list_norm.pop(3)
dash_list_ia = sns._core.unique_dashes(7)
dash_list_ia.pop(6)

dfcopy['tc_log'] = np.log(dfcopy['total_cost'])
dfia['ic_log'] = np.log(dfia['individual_cost'])

labels1 = ['Hermine [$59M]', 'Matthew [$196M]', 'Harvey [$2,362M]', 'Florence [$3M]', 'Michael [$669M]', 'Dorian [$33M]']
labels2 = ['Hermine [$9M]', 'Matthew [$16M]', 'Harvey [$1,656M]', 'Irma [$286M]', 'Florence [$55M]', 'Michael [$57M]']


dfcopy['exp'] = (np.exp(dfcopy['betas'])-1)*100
dfia['exp_ia'] = (np.exp(dfia['betas_ia'])-1)*100

#sns.set(rc={"figure.figsize":(3, 4)})
sns.set(style='white', font_scale=1.25)
fig, axes = plt.subplots(ncols=2, nrows=1, sharey=True, figsize=(9,6))
axes[0].set_xlabel('Months from hurricane')
axes[0].set_ylabel('Percent change in light')
sns.lineplot(data = dfcopy, x= 'time', y ='exp', style='hurricane',dashes=dash_list_norm,hue = 'hurricane', palette= dfp_palette,ax =axes[0], size='tc_log', legend=False)
axes[0].axhline(0, color='grey')
axes[0].tick_params(axis='x', labelrotation=0)
#axes[0].set_title('Any Aid')
axes[0].legend(title='Hurricane', labels=labels1)
axes[1].set_xlabel('Months from Hurricane')
sns.lineplot(data = dfia, x= 'time', y ='exp_ia', style='hurricane',dashes=dash_list_ia,hue = 'hurricane', palette= dfia_palette, ax =axes[1], size='ic_log', legend=False)
axes[1].axhline(0, color='grey')
#axes[1].set_title('Individual Aid')
axes[1].legend(title='Hurricane', labels=labels2)
plt.tight_layout()
plt.show()

